@@ -12,7 +12,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # clusterers.py
-# Copyright (C) 2014-2019 Fracpete (pythonwekawrapper at gmail dot com)
+# Copyright (C) 2014-2021 Fracpete (pythonwekawrapper at gmail dot com)
 
 import javabridge
 import logging
@@ -22,7 +22,7 @@ import argparse
 import traceback
 import weka.core.jvm as jvm
 import weka.core.classes as classes
-import weka.core.serialization as serialization
+from weka.core.classes import serialization_write, serialization_read_all, serialization_write_all
 from weka.core.classes import JavaObject, join_options
 from weka.core.classes import OptionHandler
 from weka.core.classes import Random
@@ -55,9 +55,17 @@ class Clusterer(OptionHandler):
         self.is_updateable = self.check_type(jobject, "weka.clusterers.UpdateableClusterer")
         self.is_drawable = self.check_type(jobject, "weka.core.Drawable")
         self.enforce_type(jobject, "weka.clusterers.Clusterer")
+        self._header = None
         super(Clusterer, self).__init__(jobject=jobject, options=options)
-        self.__cluster = javabridge.make_call(self.jobject, "clusterInstance", "(Lweka/core/Instance;)I")
-        self.__distribution = javabridge.make_call(self.jobject, "distributionForInstance", "(Lweka/core/Instance;)[D")
+
+    def _make_calls(self):
+        """
+        Method for generating instances using javabridge.make_call.
+        Members must start with "_mc_"
+        """
+        super(Clusterer, self)._make_calls()
+        self._mc_cluster = javabridge.make_call(self.jobject, "clusterInstance", "(Lweka/core/Instance;)I")
+        self._mc_distribution = javabridge.make_call(self.jobject, "distributionForInstance", "(Lweka/core/Instance;)[D")
 
     @property
     def capabilities(self):
@@ -69,6 +77,16 @@ class Clusterer(OptionHandler):
         """
         return Capabilities(javabridge.call(self.jobject, "getCapabilities", "()Lweka/core/Capabilities;"))
 
+    @property
+    def header(self):
+        """
+        Returns the header of the training data.
+
+        :return: the structure of the training data, None if not available
+        :rtype: Instances
+        """
+        return self._header
+
     def build_clusterer(self, data):
         """
         Builds the clusterer with the data.
@@ -76,6 +94,7 @@ class Clusterer(OptionHandler):
         :param data: the data to use for training the clusterer
         :type data: Instances
         """
+        self._header = data.copy_structure()
         javabridge.call(self.jobject, "buildClusterer", "(Lweka/core/Instances;)V", data.jobject)
 
     def update_clusterer(self, inst):
@@ -108,7 +127,7 @@ class Clusterer(OptionHandler):
         :return: the clustering result
         :rtype: float
         """
-        return self.__cluster(inst.jobject)
+        return self._mc_cluster(inst.jobject)
 
     def distribution_for_instance(self, inst):
         """
@@ -119,7 +138,7 @@ class Clusterer(OptionHandler):
         :return: the cluster distribution
         :rtype: float[]
         """
-        pred = self.__distribution(inst.jobject)
+        pred = self._mc_distribution(inst.jobject)
         return javabridge.get_env().get_double_array_elements(pred)
 
     @property
@@ -184,7 +203,7 @@ class Clusterer(OptionHandler):
         :rtype: tuple
         """
 
-        objs = serialization.read_all(ser_file)
+        objs = serialization_read_all(ser_file)
         if len(objs) == 1:
             return Clusterer(jobject=objs[0]), None
         elif len(objs) == 2:
@@ -207,9 +226,9 @@ class Clusterer(OptionHandler):
             header = Instances.template_instances(header)
 
         if header is not None:
-            serialization.write_all(ser_file, [self, header])
+            serialization_write_all(ser_file, [self, header])
         else:
-            serialization.write(ser_file, self)
+            serialization_write(ser_file, self)
 
 
 class SingleClustererEnhancer(Clusterer):
